@@ -4,16 +4,17 @@ _PAGE_SIZE = 100
 
 class PaginatedCollection:
     """ An iterable collection of database objects (Projects, Labels, etc...).
+
     Implements automatic (transparent to the user) paginated fetching during
     iteration. Intended for use by library internals and not by the end user.
-
     For a list of attributes see __init__(...) documentation. The params of
     __init__ map exactly to object attributes.
     """
 
     def __init__(self, client, query, params, dereferencing, obj_class):
         """ Creates a PaginatedCollection.
-        Params:
+
+        Args:
             client (labelbox.Client): the client used for fetching data from DB.
             query (str): Base query used for pagination. It must contain two
                 '%d' placeholders, the first for pagination 'skip' clause and
