@@ -3,12 +3,21 @@ from labelbox.orm.model import Field, Relationship
 
 
 class Benchmark(DbObject):
-    """ Benchmarks (also known as Golden Standard) is a quality assurance tool
-    for training data. Training data quality is the measure of accuracy and
-    consistency of the training data. Benchmarks works by interspersing data
-    to be labeled, for which there is a benchmark label, to each person labeling.
-    These labeled data are compared against their respective benchmark and an
+    """ Represents a benchmark label.
+
+    The Benchmarks tool works by interspersing data to be labeled, for
+    which there is a benchmark label, to each person labeling. These
+    labeled data are compared against their respective benchmark and an
     accuracy score between 0 and 100 percent is calculated.
+
+    Attributes:
+        created_at (datetime)
+        last_activity (datetime)
+        average_agreement (float)
+        completed_count (int)
+
+        created_by (Relationship): `ToOne` relationship to User
+        reference_label (Relationship): `ToOne` relationship to Label
     """
     created_at = Field.DateTime("created_at")
     created_by = Relationship.ToOne("User", False, "created_by")
