@@ -1,15 +1,30 @@
-import json
-from multiprocessing.dummy import Pool as ThreadPool
 import os
+import json
+import logging
+from itertools import islice
+from multiprocessing.dummy import Pool as ThreadPool
 
 from labelbox.exceptions import InvalidQueryError, ResourceNotFoundError, InvalidAttributeError
 from labelbox.orm.db_object import DbObject, Updateable, Deletable
 from labelbox.orm.model import Entity, Field, Relationship
 
+logger = logging.getLogger(__name__)
+
 
 class Dataset(DbObject, Updateable, Deletable):
-    """ A dataset is a collection of DataRows. For example, if you have a CSV with
-    100 rows, you will have 1 Dataset and 100 DataRows.
+    """ A Dataset is a collection of DataRows.
+
+    Attributes:
+        name (str)
+        description (str)
+        updated_at (datetime)
+        created_at (datetime)
+
+        projects (Relationship): `ToMany` relationship to Project
+        data_rows (Relationship): `ToMany` relationship to DataRow
+        created_by (Relationship): `ToOne` relationship to User
+        organization (Relationship): `ToOne` relationship to Organization
+
     """
     name = Field.String("name")
     description = Field.String("description")
@@ -27,12 +42,10 @@ class Dataset(DbObject, Updateable, Deletable):
 
         >>> dataset.create_data_row(row_data="http://my_site.com/photos/img_01.jpg")
 
-        Kwargs:
-            Key-value arguments containing new `DataRow` data.
-            At a minimum `kwargs` must contain `row_data`. The value for
-            `row_data` is a string. If it is a path to an existing local
-            file then it is uploaded to Labelbox's server. Otherwise it is
-            treated as an external URL.
+        Args:
+            **kwargs: Key-value arguments containing new `DataRow` data. At a minimum,
+                must contain `row_data`.
+
         Raises:
             InvalidQueryError: If `DataRow.row_data` field value is not provided
                 in `kwargs`.
@@ -61,7 +74,7 @@ class Dataset(DbObject, Updateable, Deletable):
         it is a `str`, then it is interpreted as a local file path. The file
         is uploaded to Labelbox and a DataRow referencing it is created.
         If an item is a `dict`, then it should map `DataRow` fields (or their
-        names) to values. At the minimum an `item` passed as a `dict` must 
+        names) to values. At the minimum an `item` passed as a `dict` must
         contain a `DataRow.row_data` key and value.
 
         >>> dataset.create_data_rows([
@@ -154,6 +167,33 @@ class Dataset(DbObject, Updateable, Deletable):
         task._user = user
         return task
 
+    def data_rows_for_external_id(self, external_id, limit=10):
+        """ Convenience method for getting a single `DataRow` belonging to this
+        `Dataset` that has the given `external_id`.
+
+        Args:
+            external_id (str): External ID of the sought `DataRow`.
+            limit (int): The maximum number of data rows to return for the given external_id
+
+        Returns:
+            A single `DataRow` with the given ID.
+
+        Raises:
+            labelbox.exceptions.ResourceNotFoundError: If there is no `DataRow`
+                in this `DataSet` with the given external ID, or if there are
+                multiple `DataRows` for it.
+        """
+        DataRow = Entity.DataRow
+        where = DataRow.external_id == external_id
+
+        data_rows = self.data_rows(where=where)
+        # Get at most `limit` data_rows.
+        data_rows = list(islice(data_rows, limit))
+
+        if not len(data_rows):
+            raise ResourceNotFoundError(DataRow, where)
+        return data_rows
+
     def data_row_for_external_id(self, external_id):
         """ Convenience method for getting a single `DataRow` belonging to this
         `Dataset` that has the given `external_id`.
@@ -169,14 +209,10 @@ class Dataset(DbObject, Updateable, Deletable):
                 in this `DataSet` with the given external ID, or if there are
                 multiple `DataRows` for it.
         """
-        DataRow = Entity.DataRow
-        where = DataRow.external_id == external_id
-
-        data_rows = self.data_rows(where=where)
-        # Get at most two data_rows.
-        data_rows = [row for row, _ in zip(data_rows, range(2))]
-
-        if len(data_rows) != 1:
-            raise ResourceNotFoundError(DataRow, where)
-
+        data_rows = self.data_rows_for_external_id(external_id=external_id,
+                                                   limit=2)
+        if len(data_rows) > 1:
+            logger.warning(
+                f"More than one data_row has the provided external_id : `%s`. Use function data_rows_for_external_id to fetch all",
+                external_id)
         return data_rows[0]
