@@ -1,14 +1,15 @@
+import json
+import time
+import logging
 from collections import namedtuple
 from datetime import datetime, timezone
-import json
-import logging
 from pathlib import Path
-import time
-from typing import Union, Iterable
+from typing import Dict, List, Union, Iterable
 from urllib.parse import urlparse
 
 from labelbox import utils
 from labelbox.schema.bulk_import_request import BulkImportRequest
+from labelbox.schema.data_row import DataRow
 from labelbox.exceptions import InvalidQueryError
 from labelbox.orm import query
 from labelbox.orm.db_object import DbObject, Updateable, Deletable
@@ -27,6 +28,29 @@ logger = logging.getLogger(__name__)
 class Project(DbObject, Updateable, Deletable):
     """ A Project is a container that includes a labeling frontend, an ontology,
     datasets and labels.
+
+    Attributes:
+        name (str)
+        description (str)
+        updated_at (datetime)
+        created_at (datetime)
+        setup_complete (datetime)
+        last_activity_time (datetime)
+        auto_audit_number_of_labels (int)
+        auto_audit_percentage (float)
+
+        datasets (Relationship): `ToMany` relationship to Dataset
+        created_by (Relationship): `ToOne` relationship to User
+        organization (Relationship): `ToOne` relationship to Organization
+        reviews (Relationship): `ToMany` relationship to Review
+        labeling_frontend (Relationship): `ToOne` relationship to LabelingFrontend
+        labeling_frontend_options (Relationship): `ToMany` relationship to LabelingFrontendOptions
+        labeling_parameter_overrides (Relationship): `ToMany` relationship to LabelingParameterOverride
+        webhooks (Relationship): `ToMany` relationship to Webhook
+        benchmarks (Relationship): `ToMany` relationship to Benchmark
+        active_prediction_model (Relationship): `ToOne` relationship to PredictionModel
+        predictions (Relationship): `ToMany` relationship to Prediction
+        ontology (Relationship): `ToOne` relationship to Ontology
     """
     name = Field.String("name")
     description = Field.String("description")
@@ -55,17 +79,19 @@ class Project(DbObject, Updateable, Deletable):
     ontology = Relationship.ToOne("Ontology", True)
 
     def create_label(self, **kwargs):
-        """ Creates a label on a Legacy Editor project. Not
-            supported in the new Editor.
+        """ Creates a label on a Legacy Editor project. Not supported in the new Editor.
 
-        Kwargs:
-            Label attributes. At the minimum the label `DataRow`.
+        Args:
+            **kwargs: Label attributes. At minimum, the label `DataRow`.
         """
         # Copy-paste of Client._create code so we can inject
         # a connection to Type. Type objects are on their way to being
         # deprecated and we don't want the Py client lib user to know
         # about them. At the same time they're connected to a Label at
         # label creation in a non-standard way (connect via name).
+        logger.warning(
+            "`create_label` is deprecated and is not compatible with the new editor."
+        )
 
         Label = Entity.Label
 
@@ -86,8 +112,7 @@ class Project(DbObject, Updateable, Deletable):
         return Label(self.client, res["createLabel"])
 
     def labels(self, datasets=None, order_by=None):
-        """
-        Custom relationship expansion method to support limited filtering.
+        """ Custom relationship expansion method to support limited filtering.
 
         Args:
             datasets (iterable of Dataset): Optional collection of Datasets
@@ -113,7 +138,7 @@ class Project(DbObject, Updateable, Deletable):
         id_param = "projectId"
         query_str = """query GetProjectLabelsPyApi($%s: ID!)
             {project (where: {id: $%s})
-                {labels (skip: %%d first: %%d%s%s) {%s}}}""" % (
+                {labels (skip: %%d first: %%d %s %s) {%s}}}""" % (
             id_param, id_param, where, order_by_str,
             query.results_query_part(Label))
 
@@ -129,9 +154,8 @@ class Project(DbObject, Updateable, Deletable):
         Args:
             timeout_seconds (float): Max waiting time, in seconds.
         Returns:
-            URL of the data file with this Project's labels. If the server
-                didn't generate during the `timeout_seconds` period, None
-                is returned.
+            URL of the data file with this Project's labels. If the server didn't
+            generate during the `timeout_seconds` period, None is returned.
         """
         sleep_time = 2
         id_param = "projectId"
@@ -153,6 +177,69 @@ class Project(DbObject, Updateable, Deletable):
                          self.uid)
             time.sleep(sleep_time)
 
+    def upsert_instructions(self, instructions_file: str):
+        """
+        * Uploads instructions to the UI. Running more than once will replace the instructions
+            
+        Args:
+            instructions_file (str): Path to a local file.
+                * Must be either a pdf, text, or html file.
+
+        Raises:
+            ValueError:
+                * project must be setup 
+                * instructions file must end with one of ".text", ".txt", ".pdf", ".html"
+        """
+
+        if self.setup_complete is None:
+            raise ValueError(
+                "Cannot attach instructions to a project that has not been set up."
+            )
+
+        frontend = self.labeling_frontend()
+        frontendId = frontend.uid
+
+        if frontend.name != "Editor":
+            logger.warning(
+                f"This function has only been tested to work with the Editor front end. Found %s",
+                frontend.name)
+
+        supported_instruction_formats = (".text", ".txt", ".pdf", ".html")
+        if not instructions_file.endswith(supported_instruction_formats):
+            raise ValueError(
+                f"instructions_file must end with one of {supported_instruction_formats}. Found {instructions_file}"
+            )
+
+        lfo = list(self.labeling_frontend_options())[-1]
+        instructions_url = self.client.upload_file(instructions_file)
+        customization_options = json.loads(lfo.customization_options)
+        customization_options['projectInstructions'] = instructions_url
+        option_id = lfo.uid
+
+        self.client.execute(
+            """mutation UpdateFrontendWithExistingOptionsPyApi (
+                    $frontendId: ID!, 
+                    $optionsId: ID!, 
+                    $name: String!, 
+                    $description: String!, 
+                    $customizationOptions: String!
+                ) {
+                    updateLabelingFrontend(
+                        where: {id: $frontendId}, 
+                        data: {name: $name, description: $description}
+                    ) {id}
+                    updateLabelingFrontendOptions(
+                        where: {id: $optionsId}, 
+                        data: {customizationOptions: $customizationOptions}
+                    ) {id}
+                }""", {
+                "frontendId": frontendId,
+                "optionsId": option_id,
+                "name": frontend.name,
+                "description": "Video, image, and text annotation",
+                "customizationOptions": json.dumps(customization_options)
+            })
+
     def labeler_performance(self):
         """ Returns the labeler performances for this Project.
 
@@ -186,7 +273,7 @@ class Project(DbObject, Updateable, Deletable):
         Args:
             net_score (None or Review.NetScore): Indicates desired metric.
         Returns:
-            int, aggregation count of reviews for given net_score.
+            int, aggregation count of reviews for given `net_score`.
         """
         if net_score not in (None,) + tuple(Entity.Review.NetScore):
             raise InvalidQueryError(
@@ -229,18 +316,66 @@ class Project(DbObject, Updateable, Deletable):
         timestamp = datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%SZ")
         self.update(setup_complete=timestamp)
 
-    def set_labeling_parameter_overrides(self, data):
-        """ Adds labeling parameter overrides to this project. Example:
+    def validate_labeling_parameter_overrides(self, data):
+        for idx, row in enumerate(data):
+            if len(row) != 3:
+                raise TypeError(
+                    f"Data must be a list of tuples containing a DataRow, priority (int), num_labels (int). Found {len(row)} items. Index: {idx}"
+                )
+            data_row, priority, num_labels = row
+            if not isinstance(data_row, DataRow):
+                raise TypeError(
+                    f"data_row should be be of type DataRow. Found {type(data_row)}. Index: {idx}"
+                )
 
+            for name, value in [["Priority", priority],
+                                ["Number of labels", num_labels]]:
+                if not isinstance(value, int):
+                    raise TypeError(
+                        f"{name} must be an int. Found {type(value)} for data_row {data_row}. Index: {idx}"
+                    )
+                if value < 1:
+                    raise ValueError(
+                        f"{name} must be greater than 0 for data_row {data_row}. Index: {idx}"
+                    )
+
+    def set_labeling_parameter_overrides(self, data):
+        """ Adds labeling parameter overrides to this project.
+                
+        See information on priority here:
+            https://docs.labelbox.com/en/configure-editor/queue-system#reservation-system
+    
             >>> project.set_labeling_parameter_overrides([
             >>>     (data_row_1, 2, 3), (data_row_2, 1, 4)])
 
         Args:
             data (iterable): An iterable of tuples. Each tuple must contain
-                (DataRow, priority, numberOfLabels) for the new override.
+                (DataRow, priority<int>, number_of_labels<int>) for the new override.
+
+                Priority:
+                    * Data will be labeled in priority order.
+                        - A lower number priority is labeled first.
+                        - Minimum priority is 1.
+                    * Priority is not the queue position.
+                        - The position is determined by the relative priority.
+                        - E.g. [(data_row_1, 5,1), (data_row_2, 2,1), (data_row_3, 10,1)] 
+                            will be assigned in the following order: [data_row_2, data_row_1, data_row_3]
+                    * Datarows with parameter overrides will appear before datarows without overrides.
+                    * The priority only effects items in the queue.
+                        - Assigning a priority will not automatically add the item back into the queue.  
+                Number of labels:
+                    * The number of times a data row should be labeled.
+                        - Creates duplicate data rows in a project (one for each number of labels).
+                    * New duplicated data rows will be added to the queue.
+                        - Already labeled duplicates will not be sent back to the queue.
+                    * The queue will never assign the same datarow to a single labeler more than once.
+                        - If the number of labels is greater than the number of labelers working on a project then
+                            the extra items will remain in the queue (this can be fixed by removing the override at any time).
+                    * Setting this to 1 will result in the default behavior (no duplicates).
         Returns:
             bool, indicates if the operation was a success.
         """
+        self.validate_labeling_parameter_overrides(data)
         data_str = ",\n".join(
             "{dataRow: {id: \"%s\"}, priority: %d, numLabels: %d }" %
             (data_row.uid, priority, num_labels)
@@ -255,6 +390,8 @@ class Project(DbObject, Updateable, Deletable):
     def unset_labeling_parameter_overrides(self, data_rows):
         """ Removes labeling parameter overrides to this project.
 
+        * This will remove unlabeled duplicates in the queue.
+
         Args:
             data_rows (iterable): An iterable of DataRows.
         Returns:
@@ -270,12 +407,19 @@ class Project(DbObject, Updateable, Deletable):
         return res["project"]["unsetLabelingParameterOverrides"]["success"]
 
     def upsert_review_queue(self, quota_factor):
-        """ Reinitiates the review queue for this project.
+        """ Sets the the proportion of total assets in a project to review.
+
+        More information can be found here: 
+            https://docs.labelbox.com/en/quality-assurance/review-labels#configure-review-percentage
 
         Args:
             quota_factor (float): Which part (percentage) of the queue
                 to reinitiate. Between 0 and 1.
         """
+
+        if not 0. < quota_factor < 1.:
+            raise ValueError("Quota factor must be in the range of [0,1]")
+
         id_param = "projectId"
         quota_param = "quotaFactor"
         query_str = """mutation UpsertReviewQueuePyApi($%s: ID!, $%s: Float!){
@@ -290,7 +434,6 @@ class Project(DbObject, Updateable, Deletable):
     def extend_reservations(self, queue_type):
         """ Extends all the current reservations for the current user on the given
         queue type.
-
         Args:
             queue_type (str): Either "LabelingQueue" or "ReviewQueue"
         Returns:
@@ -307,14 +450,19 @@ class Project(DbObject, Updateable, Deletable):
         return res["extendReservations"]
 
     def create_prediction_model(self, name, version):
-        """ Creates a PredictionModel connected to a Legacy Editor
-            Project.
+        """ Creates a PredictionModel connected to a Legacy Editor Project.
+
         Args:
             name (str): The new PredictionModel's name.
             version (int): The new PredictionModel's version.
-        Return:
+        Returns:
             A newly created PredictionModel.
         """
+
+        logger.warning(
+            "`create_prediction_model` is deprecated and is not compatible with the new editor."
+        )
+
         PM = Entity.PredictionModel
         model = self.client._create(PM, {
             PM.name.name: name,
@@ -324,8 +472,9 @@ class Project(DbObject, Updateable, Deletable):
         return model
 
     def create_prediction(self, label, data_row, prediction_model=None):
-        """ Creates a Prediction within a Legacy Editor Project.
-        
+        """ Creates a Prediction within a Legacy Editor Project. Not supported
+        in the new Editor.
+
         Args:
             label (str): The `label` field of the new Prediction.
             data_row (DataRow): The DataRow for which the Prediction is created.
@@ -339,6 +488,10 @@ class Project(DbObject, Updateable, Deletable):
                 is None and this Project's active_prediction_model is also
                 None.
         """
+        logger.warning(
+            "`create_prediction` is deprecated and is not compatible with the new editor."
+        )
+
         if prediction_model is None:
             prediction_model = self.active_prediction_model()
             if prediction_model is None:
@@ -367,52 +520,51 @@ class Project(DbObject, Updateable, Deletable):
         res = self.client.execute(query_str, params)
         return Prediction(self.client, res["createPrediction"])
 
-    def enable_model_assisted_labeling(self, toggle: bool=True) -> bool:
+    def enable_model_assisted_labeling(self, toggle: bool = True) -> bool:
         """ Turns model assisted labeling either on or off based on input
 
         Args:
-            toggle (Boolean): True or False boolean 
+            toggle (bool): True or False boolean
         Returns:
             True if toggled on or False if toggled off
         """
-
         project_param = "project_id"
         show_param = "show"
 
         query_str = """mutation toggle_model_assisted_labelingPyApi($%s: ID!, $%s: Boolean!) {
             project(where: {id: $%s }) {
-                showPredictionsToLabelers(show: $%s) { 
+                showPredictionsToLabelers(show: $%s) {
                     id, showingPredictionsToLabelers
                 }
             }
-        }""" % (project_param, show_param,project_param, show_param)
+        }""" % (project_param, show_param, project_param, show_param)
 
-        params = {
-            project_param: self.uid, 
-            show_param: toggle
-        }
+        params = {project_param: self.uid, show_param: toggle}
 
         res = self.client.execute(query_str, params)
-        return res["project"]["showPredictionsToLabelers"]["showingPredictionsToLabelers"]
+        return res["project"]["showPredictionsToLabelers"][
+            "showingPredictionsToLabelers"]
 
     def upload_annotations(
-        self,
-        name: str,
-        annotations: Union[str, Union[str, Path], Iterable[dict]],
-    ) -> 'BulkImportRequest':  # type: ignore
+            self,
+            name: str,
+            annotations: Union[str, Path, Iterable[Dict]],
+            validate: bool = True) -> 'BulkImportRequest':  # type: ignore
         """ Uploads annotations to a new Editor project.
 
         Args:
-            name: name of the BulkImportRequest job
-            annotations:
+            name (str): name of the BulkImportRequest job
+            annotations (str or Path or Iterable):
                 url that is publicly accessible by Labelbox containing an
                 ndjson file
                 OR local path to an ndjson file
                 OR iterable of annotation rows
+            validate (bool):
+                Whether or not to validate the payload before uploading.
         Returns:
             BulkImportRequest
-
         """
+
         if isinstance(annotations, str) or isinstance(annotations, Path):
 
             def _is_url_valid(url: Union[str, Path]) -> bool:
@@ -430,12 +582,11 @@ class Project(DbObject, Updateable, Deletable):
                 return bool(parsed.scheme) and bool(parsed.netloc)
 
             if _is_url_valid(annotations):
-                return BulkImportRequest.create_from_url(
-                    client=self.client,
-                    project_id=self.uid,
-                    name=name,
-                    url=str(annotations),
-                )
+                return BulkImportRequest.create_from_url(client=self.client,
+                                                         project_id=self.uid,
+                                                         name=name,
+                                                         url=str(annotations),
+                                                         validate=validate)
             else:
                 path = Path(annotations)
                 if not path.exists():
@@ -447,7 +598,7 @@ class Project(DbObject, Updateable, Deletable):
                     project_id=self.uid,
                     name=name,
                     file=path,
-                    validate_file=True,
+                    validate_file=validate,
                 )
         elif isinstance(annotations, Iterable):
             return BulkImportRequest.create_from_objects(
@@ -455,12 +606,19 @@ class Project(DbObject, Updateable, Deletable):
                 project_id=self.uid,
                 name=name,
                 predictions=annotations,  # type: ignore
-            )
+                validate=validate)
         else:
             raise ValueError(
                 f'Invalid annotations given of type: {type(annotations)}')
 
+
 class LabelingParameterOverride(DbObject):
+    """ Customizes the order of assets in the label queue.
+
+    Attributes:
+        priority (int): A prioritization score.
+        number_of_labels (int): Number of times an asset should be labeled.
+    """
     priority = Field.Int("priority")
     number_of_labels = Field.Int("number_of_labels")
 
