@@ -5,6 +5,21 @@ from labelbox.orm.model import Field, Relationship
 class User(DbObject):
     """ A User is a registered Labelbox user (for example you) associated with
     data they create or import and an Organization they belong to.
+
+    Attributes:
+        updated_at (datetime)
+        created_at (datetime)
+        email (str)
+        name (str)
+        nickname (str)
+        intercom_hash (str)
+        picture (str)
+        is_viewer (bool)
+        is_external_viewer (bool)
+
+        organization (Relationship): `ToOne` relationship to Organization
+        created_tasks (Relationship): `ToMany` relationship to Task
+        projects (Relationship): `ToMany` relationship to Project
     """
     updated_at = Field.DateTime("updated_at")
     created_at = Field.DateTime("created_at")
